#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import argparse
import json
import sys

from .api import PyBGPRanking

from urllib.parse import urljoin
from pyipasnhistory import IPASNHistory
from datetime import date, timedelta


def main():
    parser = argparse.ArgumentParser(description='Run a query against BGP Ranking')
    parser.add_argument('--url', type=str, help='URL of the instance.')
    parser.add_argument('--date', default=date.today().isoformat(), help='Date of the dataset required')

    sub_parsers = parser.add_subparsers(title='Available commands', dest='command')

    index_query = sub_parsers.add_parser('index')
    index_query.add_argument('--limit', default=100, help='Max number of ASN to get')
    index_query.add_argument('--family', default='v4', help='v4 or v6')
    index_query.set_defaults(which='index')

    simple_query = sub_parsers.add_parser('simple')
    group = simple_query.add_mutually_exclusive_group(required=True)
    group.add_argument('--asn', help='ASN to lookup')
    group.add_argument('--ip', help='IP to lookup')
    simple_query.set_defaults(which='simple')

    status_query = sub_parsers.add_parser('status')

    group = status_query.add_mutually_exclusive_group(required=True)
    group.add_argument('--redis_up', action='store_true', help='Check if redis is up.')

    args = parser.parse_args()

    if args.url:
        bgpranking = PyBGPRanking(args.url)
        ipasn = IPASNHistory(urljoin(args.url, 'ipasn_history'))
    else:
        bgpranking = PyBGPRanking()
        ipasn = IPASNHistory()

    if not bgpranking.is_up:
        print(f'Unable to reach {bgpranking.root_url}. Is the server up?')
        sys.exit(1)

    if args.command == 'simple':
        if args.ip:
            response = ipasn.query(args.ip)
            print(json.dumps(response, indent=2))
            if 'response' in response and response['response']:
                asn = response['response'][list(response['response'].keys())[0]]['asn']
        else:
            asn = args.asn

        response = bgpranking.query(asn, date=(date.today() - timedelta(1)).isoformat())
    elif args.command == 'index':
        response = bgpranking.asns_global_ranking(address_family=args.family, limit=args.limit, date=args.date)
    elif args.command == 'status':
        if args.redis_up:
            response = bgpranking.redis_up()

    print(json.dumps(response, indent=2))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from datetime import date
from typing import Dict, Union, Any
from urllib.parse import urljoin, urlparse

import requests


class PyBGPRanking():

    def __init__(self, root_url: str='https://bgpranking-ng.circl.lu/'):
        '''Query a specific instance.

        :param root_url: URL of the instance to query.
        '''
        self.root_url = root_url

        if not urlparse(self.root_url).scheme:
            self.root_url = 'http://' + self.root_url
        if not self.root_url.endswith('/'):
            self.root_url += '/'
        self.session = requests.session()

    @property
    def is_up(self) -> bool:
        '''Test if the given instance is accessible'''
        r = self.session.head(self.root_url)
        return r.status_code == 200

    def redis_up(self) -> Dict:
        '''Check if redis is up and running'''
        r = self.session.get(urljoin(self.root_url, 'redis_up'))
        return r.json()

    def query(self, asn: str, address_family: str='v4', date: str=None,
              source: Union[list, str]=''):
        '''Launch a query.
        :param asn: ASN to lookup
        :param address_family: v4 or v6
        :param date: Exact date to lookup. Fallback to most recent available.
        :param source: Source to query. Can be a list of sources.
        '''
        to_query: Dict[str, Any] = {'asn': asn, 'address_family': address_family}
        if date:
            to_query['date'] = date
        if source:
            to_query['source'] = source
        r = self.session.post(urljoin(self.root_url, '/json/asn'), json=to_query)
        return r.json()

    def asns_global_ranking(self, date: str=date.today().isoformat(), address_family: str='v4', limit: int=100):
        '''Get the top `limit` ASNs, from worse to best'''
        to_query = {'date': date, 'ipversion': address_family, 'limit': limit}
        r = self.session.post(urljoin(self.root_url, '/json/asns_global_ranking'), json=to_query)
        return r.json()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pybgpranking2']

package_data = \
{'': ['*']}

install_requires = \
['pyipasnhistory>=2.1,<3.0', 'requests>=2.26.0,<3.0.0']

extras_require = \
{'docs': ['Sphinx>=4.3,<5.0']}

entry_points = \
{'console_scripts': ['bgpranking = pybgpranking2:main']}

setup_kwargs = {
    'name': 'pybgpranking2',
    'version': '2.0.1',
    'description': 'Python CLI and module for BGP Ranking',
    'long_description': '# Python client and module for BGP Ranking\n\nTHis project will make querying BGP Ranking easier.\n\n## Installation\n\n```bash\npip install pybgpranking\n```\n\n## Usage\n\n### Command line\n\nYou can use the `bgpranking` command to query the instance:\n\n```bash\n```\n\n### Library\n\nSee [API Reference]()\n',
    'author': 'Raphaël Vinot',
    'author_email': 'raphael.vinot@circl.lu',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
